//----------------------------------------------------------------------------------------------
//	$Id: frogs.js,v 1.2 2023/08/22 22:46:47 wolf Exp $
//----------------------------------------------------------------------------------------------
//
//	Bewegt den Frosch und berechnet die Statistik
//
//----------------------------------------------------------------------------------------------

/*jsl:option explicit*/

'use strict';

(function(){

	//--------------------------------------------------------------------------------------
	//	installiere die Funktionen in einem Container
	//--------------------------------------------------------------------------------------

	function install( container ){

		//------------------------------------------------------------------------------
		//	SVG element
		//------------------------------------------------------------------------------

		const svgElement = container.getElementsByTagName( 'svg' )[0];
		if( !svgElement ){ return; }

		//------------------------------------------------------------------------------
		//	lese Konfiguration aus Element
		//------------------------------------------------------------------------------

		const jumpslow = parseFloat( svgElement.dataset.jumpslow ) || 500;
		const jumpfast = parseFloat( svgElement.dataset.jumpfast ) || 50;
		const timestep = parseFloat( svgElement.dataset.timestep ) || 20;

		//------------------------------------------------------------------------------
		//	Bewegliches Element (Frosch)
		//------------------------------------------------------------------------------

		const movableElement = svgElement.getElementsByClassName( 'frogs' )[ 0 ];

		//------------------------------------------------------------------------------
		//	lese Konfiguration aus Element
		//------------------------------------------------------------------------------

		const baseX    = parseFloat( movableElement.dataset.baseX  ) || 0;
		const baseY    = parseFloat( movableElement.dataset.baseY  ) || 0;
		const distX    = parseFloat( movableElement.dataset.distX  ) || 0;
		const distY    = parseFloat( movableElement.dataset.distY  ) || 0;
		const eleY     = parseFloat( movableElement.dataset.eleY   ) || 0;
		const maxPos   = parseFloat( movableElement.dataset.maxPos ) || 10;

		//------------------------------------------------------------------------------
		//	Eingabeformular
		//------------------------------------------------------------------------------

		const formElement = container.getElementsByTagName( 'form' )[0];

		formElement.onsubmit = function(){ return false; };

		const buttonHops   = formElement.elements.hops  || {};
		const buttonReset  = formElement.elements.reset || {};
		const checkboxAuto = formElement.elements.auto  || {};
		const checkboxFast = formElement.elements.fast  || {};
		const checkboxNoAni= formElement.elements.noani || {};
		const inputNhops   = formElement.elements.nhops || {};
		const inputStart   = formElement.elements.start || {};

		//------------------------------------------------------------------------------
		//	Statistik
		//------------------------------------------------------------------------------

		const nusedCounts   = [{}];
		const nusedShares   = [{}];
		const visitCounts   = [];
		const visitPercents = [];
		const nusedAverages = [];

		const tdElements = container.getElementsByTagName( 'td' );
		for( var tdIndex = 0; tdIndex < tdElements.length; ++tdIndex ){

			const tdElement = tdElements[ tdIndex ];

			const nc = tdElement.getAttribute( 'data-nused-count' );
			if( nc ){ nusedCounts[ parseInt(nc, 10) ] = tdElement; }

			const ns = tdElement.getAttribute( 'data-nused-share' );
			if( ns ){ nusedShares[ parseInt(ns, 10) ] = tdElement; }

			const vc = tdElement.getAttribute( 'data-visit-count' );
			if( vc ){ visitCounts[ parseInt(vc, 10) ] = tdElement; }

			const vp = tdElement.getAttribute( 'data-visit-percent' );
			if( vp ){ visitPercents[ parseInt(vp, 10) ] = tdElement; }

			const name = tdElement.getAttribute( 'data-name' );
			if( name=='nused-average' ){ nusedAverages.push( tdElement ); }
		}

		const visitedFields = [];

		//------------------------------------------------------------------------------
		//	bewege den Frosch graphisch auf (auch interpolierte) Positionen
		//------------------------------------------------------------------------------

		function unconditionallyMoveTo0(){

			movableElement.setAttributeNS( null, 'x', baseX );
			movableElement.setAttributeNS( null, 'y', baseY );
		}

		function moveTo( pos, phase ){

			if( checkboxNoAni.checked && checkboxAuto.checked ){ return; }

			const x = baseX + distX * pos;
			const y = baseY + 0 * pos + eleY * (Math.sin( Math.PI * phase) || 0);

			movableElement.setAttributeNS( null, 'x', x );
			movableElement.setAttributeNS( null, 'y', y );
		}

		//------------------------------------------------------------------------------
		//	timer reset
		//------------------------------------------------------------------------------

		var timer   = null;
		var timeout = null;

		function stop(){

			window.clearInterval( timer   );
			window.clearTimeout ( timeout );
		}

		//------------------------------------------------------------------------------
		//	aktuelle ganzzahlige Position
		//------------------------------------------------------------------------------

		var currentPosition = 0;

		//------------------------------------------------------------------------------
		//	reset
		//------------------------------------------------------------------------------

		function reset(){

			stop();
			currentPosition = 0;
			moveTo( maxPos - 0.5 );

			for( var element of nusedCounts   ){ element.innerText = 0     ; }
			for(     element of nusedShares   ){ element.innerText = '\xa0'; }
			for(     element of visitCounts   ){ element.innerText = 0     ; }
			for(     element of visitPercents ){ element.innerText = '\xa0'; }
			for(     element of nusedAverages ){ element.innerText = '\xa0'; }
		}

		reset();

		//------------------------------------------------------------------------------
		//	beginne neue Sprungsequenz
		//------------------------------------------------------------------------------

		function resetSequence(){

			stop();
			currentPosition = inputStart.value;
			moveTo( currentPosition );

			visitedFields.splice(0);

			if( !checkboxAuto.checked ){ return; }

			const duration = checkboxNoAni.checked ? 0 :
						checkboxFast.checked ? jumpfast : jumpslow;

			timeout = window.setTimeout(function(){

				animatedGoto();
			}, duration );
		}

		//------------------------------------------------------------------------------
		//	neue Position erreicht
		//------------------------------------------------------------------------------

		function inc( element ){

			return element.innerText = parseInt(element.innerText, 10) + 1;
		}

		function reached( pos ){

			currentPosition = pos;
			moveTo( currentPosition );

			//----------------------------------------------------------------------
			//	aktualisiere Statistik
			//----------------------------------------------------------------------

			visitedFields.push( currentPosition );

			const base = inc( visitCounts[ currentPosition ]);

			if( currentPosition === 0 ){

				for( var v in visitCounts ){

					const vc = parseInt( visitCounts[v].innerText, 10 );
					if( !vc ){ continue; }
					const percent = vc / base * 100;
					visitPercents[v].innerText = percent.toFixed( 2 );
				}

				inc( nusedCounts[ visitedFields.length ] );

				var sum = 0;

				for( var u in nusedCounts ){

					const uc = parseInt( nusedCounts[u].innerText, 10 );
					if( !uc ){ continue; }
					sum += u * uc;
					const share = uc / base;
					nusedShares[u].innerText = share.toFixed( 8 );
				}

				for( var average of nusedAverages ){

					average.innerText = (sum/base).toFixed( 4 );
				}
			}

			//----------------------------------------------------------------------
			//	automatisch weiterhuepfen
			//----------------------------------------------------------------------

			if( checkboxAuto.checked ){

				if( currentPosition ){ animatedGoto(); return; }

				const count = Math.max( 0, (inputNhops.value - 1) || 0 );
				inputNhops.value = count || '';

				if( count ){

					const duration = checkboxNoAni.checked ? 0 :
						checkboxFast.checked ? jumpfast : jumpslow;
					timeout = window.setTimeout( resetSequence, duration );
				}

				if( !count ){ unconditionallyMoveTo0(); }
			}
		}

		//------------------------------------------------------------------------------
		//	springe animiert auf neue Position
		//------------------------------------------------------------------------------

		function animatedGoto(){

			const oldPosition = currentPosition;
			const newPosition = Math.floor( Math.random() * currentPosition );

			if( checkboxNoAni.checked ){ reached( newPosition ); return; }

			const duration = checkboxFast.checked ? jumpfast : jumpslow;

			var time = 0;
			timer = window.setInterval( function(){

				time = Math.min( time + timestep, duration );

				if( time < duration ){

					const k = time/duration;
					moveTo( oldPosition * (1-k) + newPosition * k, k );
					return;
				}

				//--------------------------------------------------------------
				//	neue Position erreicht
				//--------------------------------------------------------------

				stop();
				reached( newPosition );

			}, timestep );
		}

		//------------------------------------------------------------------------------
		//	ein Huepfer
		//------------------------------------------------------------------------------

		function hop(){

			stop();

			if( currentPosition ){

				animatedGoto();
			} else {
				resetSequence();
			}
		}

		//------------------------------------------------------------------------------
		//	Ruecksetzen bei Aenderung des Startfeldes oder Ruecksetzknopf
		//------------------------------------------------------------------------------

		inputStart.onchange = reset;
		buttonReset.onclick = buttonReset.ontouchstart = reset;

		//------------------------------------------------------------------------------
		//	starte animierte Sequenz
		//------------------------------------------------------------------------------

		buttonHops.onclick = buttonHops.ontouchstart = hop;
	}

	//--------------------------------------------------------------------------------------
	//	Container sind mit 'class="frogs"' markiert
	//--------------------------------------------------------------------------------------

	const containers = document.getElementsByClassName( 'frogs' );

	for( var e = 0; e<containers.length; ++e ){

		install( containers[e] );
	}
})();

//----------------------------------------------------------------------------------------------
//	$Id: frogs.js,v 1.2 2023/08/22 22:46:47 wolf Exp $
//----------------------------------------------------------------------------------------------
